//=========================DOKUMENTACNI CAST==================================

  /*
   * CH_HLAV.C, verze 1.0
   *
   * Hlavni modul programu
   * Obsluhuje beh programu, vola sdilene fce a osetruje vstupy a vystupy
   *
   * Martin Slouf, 22.4.1999
   *
   */

//========================POTREBNE INCLUDE SOUBORY============================

  /*
   * systemove .H soubory #include <stdio.h>
   *
   * vlastni .H soubory #include "vzor.h"
   *
   */

#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <stdlib.h>
#include "ch_tab.h"
#include "ch_keyb.h"
#include "ch_vyp.h"

//===================DEFINICE GLOBALNICH IMPORTOVANYCH OBJEKTU================

  //zadne by byt nemely - vse by melo byt v .H souborech

  /*
   * jde o importovane objekty (fce nebo promenne) - jiz musi byt v nejakem
   * modulu definovany (v casti globalnich definic)
   *
   * zde se objevi jako extern deklarace
   *
   */

//=================DEFINICE GLOBALNICH EXPORTOVANYCH PROMENNYCH===============

  /*
   * jde o globalni promenne, ktere maji byt sdileny s jinymi moduly, zde
   * nejsou definovany jako extern - jde o jejich "domaci" modul
   *
   * ve sdilenych modulech se objevi za pomoci .H souboru prislusejiciho
   * k tomuto modulu (v tomto .H souboru budou uvedeny jako extern deklarace)
   *
   */

//=======================LOKALNI DEFINICE PRO TENTO MODUL=====================

  /*
   * lokalni definice pouze pro tento modul (soubor), pokud by tyto definice
   * byly prilis rozsahle -> specialni .H soubor
   *
   */

  /*
   * 1. symbolicke konstanty a makra s parametry
   *
   */

#define MAXMENU 10
  /* maximalni pocet polozek v menu je MAXMENU - 1
   * posledni polozka (na pozici MAXMENU) musi byt prazdny retez ""
   * obdobne posledni zkratkova klavesa musi byt '\0'
   * a posledni ukazatel na fci by mel byt NULL
   * pouzito v deklaraci typu MENU
   */

#define RADKA 80
  /* maximalni normalni pocet znaku na radce
   * pouzito pro nacitani vzorce ve fcich menu
   *         1. fce v_dx
   *         2. fce vlastnosti
   */

#define DX 8
  /* pocet provedenych vypoctu pro dx */

#define STRANKOVANI 20
  /* pocet radku na strance ve vypisu setridenych prvku tabulky
   * pouzito ve fcich t_podle_????
   */

/* barvy pro vypis */
#define NORMAL  7
#define VYSTUP 15

  /*
   * 2. typy - zasadne pomoci typedef
   *
   */

/* TYP MENU */
typedef struct {
  char *polozky[MAXMENU];
  P_FV fce[MAXMENU];
  int zkratky[MAXMENU];
} MENU, *P_MENU;

  /*
   * Nasleduji funkcni prototypy fci, ktere jsou vyuzity k definici
   * jednotlivych menu - predstavuji obsluhu volby nabidky
   *
   */

/* fce pro menu hlavni */
static void vypocty(void);
static void trideni(void);
static void vlastnosti(void);
static void presmerovani(void);
static void napoveda(void);
static void konec(void);
static void zpet(void);

/* fce pro menu vyp (vypocty) */
static void v_mr(void);
static void v_dx(void);

/* fce pro menu trid (trideni) */
static void t_podle_znacky(void);
static void t_podle_mr(void);
static void t_podle_z(void);
static void t_podle_chi(void);

/* fce pro menu presmer (presmerovani) */
static void pr_na_obrazovku(void);
static void pr_do_souboru(void);

  /*
   * 3. promenne - jde o globalni promenne pro tento modul - v pametove tride
   *    static
   *
   */

/* MENU HLAVNI */
static MENU hlavni = {
                       {"[F2]  VYPOCTY",
                        "[F3]  TRIDENI",
                        "[F4]  VYPIS VLASTNOSTI",
                        "[F5]  PRESMEROVANI VYSTUPU",
                        "[F1]  NAPOVEDA", "[ESC] KONEC", ""} ,
                       {vypocty, trideni, vlastnosti, presmerovani,
                        napoveda, konec, NULL} ,
                       {F2, F3, F4, F5, F1, ESC, '\0'}
                     };
static P_MENU p_hlavni = &hlavni;

/* MENU VYPOCTY */
static MENU vyp = {
                    {"[F2]  VYPOCET MOLARNI HMOTNOSTI",
                     "[F3]  VYPOCET KRYSTALOGRAFICKE HUSTOTY",
                     "[F1]  NAPOVEDA", "[ESC] ZPET", ""} ,
                    {v_mr, v_dx,
		     napoveda, zpet, NULL} ,
                    {F2, F3, F1, ESC, '\0'}
                  };
static P_MENU p_vyp = &vyp;

/* MENU VYP_OBJEM */
  /* toto menu ma jednu zvlastnost - neni definovanaa volba zpet a napoveda
   * navic se trochu lisi i pri zpracovani - nedochazi k volani fci - viz kod
   * fce v_dx
   */
static MENU vyp_objem = {
                          {"[F2] KUBICKA",
		           "[F3] TETRAGONALNI",
			   "[F4] ORTOROMBICKA",
                           "[F5] HEXAGONALNI",
                           "[F6] ROMBOEDRICKA",
                           "[F7] MONOKLINICKA",
                           "[F8] TRIKLINICKA",
                           "[F9] ZADAM OBJEM",
                           ""} ,
			  {stisk_F2, stisk_F3, stisk_F4, stisk_F5, stisk_F6,
			   stisk_F7, stisk_F8, stisk_F9,
			   NULL} ,
			  {F2, F3, F4, F5, F6, F7, F8, F9, '\0'}
                        };
static P_MENU p_vyp_objem = &vyp_objem;

/* MENU TRIDENI */
static MENU trid = {
                     {"[F2]  PODLE ZNACKY",
		      "[F3]  PODLE MOLARNI HMOTNOSTI",
		      "[F4]  PODLE PROTONOVEHO CISLA",
		      "[F5]  PODLE ELEKTRONEGATIVITY",
		      "[F1]  NAPOVEDA", "[ESC] ZPET", ""} ,
		     {t_podle_znacky, t_podle_mr, t_podle_z, t_podle_chi,
		      napoveda, zpet, NULL} ,
		     {F2, F3, F4, F5, F1, ESC, '\0'}
                   };
static P_MENU p_trid = &trid;

/* MENU TRID_VYPIS */
  /* toto menu ma jednu zvlastnost - neni definovanaa volba zpet a napoveda
   * navic se trochu lisi i pri zpracovani - nedochazi k volani fci - viz kod
   * fce t_????
   */
static MENU trid_vypis = {
                           {"[PGUP] ",
		            "[PGDN] ",
		            "[ESC]  UKONCIT PROHLIZENI", ""} ,
		           {stisk_PGUP, stisk_PGDN, stisk_ESC, NULL} ,
		           {PGUP, PGDN, ESC, '\0'}
                         };
static P_MENU p_trid_vypis = &trid_vypis;

/* MENU PRESMEROVANI */
static MENU presmer = {
                        {"[F2]  OBRAZOVKA",
                         "[F3]  UZIVATELEM ZADANY SOUBOR",
		         "[F1]  NAPOVEDA", "[ESC] ZPET", ""} ,
                        {pr_na_obrazovku, pr_do_souboru,
                         napoveda, zpet, NULL} ,
                        {F2, F3, F1, ESC} ,
                      };
static P_MENU p_presmer = &presmer;

/* VYSTUP */
FILE *vystup = stdout;
  /* urcuje soubor pro vypis informaci */

  /*
   * 4. uplne funkcni prototypy lokalnich (= pro tento modul) funkci, tj.
   * v pametove tride static
   *
   */

static void vypis_menu(P_MENU menu);
static void zpracuj_menu(P_MENU menu);

  /*
   * 5. fce main - pokud jde o hlavni soubor projektu
   *
   */

//-------------------------------MAIN-----------------------------------------
int main(void)
{
  /* nacteni hodnot */
  if (nacti_data(&tabulka) == 0)
    printf("\nERROR: DATA NEBYLA NACTENA...\n");
  else { /* data byla nactena */
    printf("\nOK: DATA NACTENA...\n");

    /* telo programu */
    for (; ;) {
      clrscr();
      vypis_menu(p_hlavni);
      zpracuj_menu(p_hlavni);
    }
  }
  return(0);
}

//========================DEFINICE GLOBALNICH FUNKCI==========================

  /*
   * jde o funkce, ktere jsou exportem modulu, jejich uplne funkcni prototypy
   * jsou v prislusnem .H souboru (obdobne jako u globalnich promennych, ani
   * zde neni uvedena zadna pametova trida, ale v prislusnem .H souboru jsou
   * deklarovany jako extern)
   *
   */

//========================DEFINICE LOKALNICH FUNKCI===========================

  /*
   * definice funkci pro tento modul (pametova trida static)
   *
   * jejich funkcni prototypy - viz vyse
   *
   */

  /*
   * 1. Fce pro obsluhu menu
   *
   * 2. Obsluhy udalosti -> hlavni menu + obecna fce zpet
   *
   * 3. Obsluhy udalosti -> menu vypocty
   *
   * 4. Obsluhy udalosti -> menu trideni
   *
   * 5. Obsluhy udalosti -> menu presmerovani
   *
   */

  /***************************
   *                         *
   * 1. Fce pro obsluhu menu *
   *                         *
   ***************************/

//---------------------------VYPIS_MENU---------------------------------------
static void vypis_menu(P_MENU menu)
{
  int i;

  _setcursortype(_NOCURSOR); /* vypnuti kurzoru */

  for (i = 0; (strcmp(menu->polozky[i], "") != 0); ++i)
    printf("%s\n", menu->polozky[i]);
}

//--------------------------ZPRACUJ_MENU--------------------------------------
static void zpracuj_menu(P_MENU menu)
{
  int i;
  int key;

  key = getdefined(menu->zkratky, &i);

  /* oznaceni volby */
  printf("\n"
         "\tVolba: %s\n"
         "\n", menu->polozky[i]);

  menu->fce[i]();
    /* proc nelze : (*menu).(*(fce + i))(); (podle K&R)
     * nebo: menu->(*(fce +i))(); (podle ANSI C)
     */
}

  /***************************************
   *                                     *
   * 2. Obsluhy udalosti  -> hlavni menu *
   *                                     *
   ***************************************/

//-----------------------VYPOCTY----------------------------------------------
static void vypocty(void)
{
  vypis_menu(p_vyp);
  zpracuj_menu(p_vyp);
}

//-------------------------TRIDENI---------------------------------------------
static void trideni(void)
{
  vypis_menu(p_trid);
  zpracuj_menu(p_trid);
}

//--------------------------VLASTNOSTI-----------------------------------------
static void vlastnosti(void)
{
  char znacka[RADKA];
  P_PRVEK atom;

  _setcursortype(_NORMALCURSOR); /* zobrazeni kurzoru */

  /* zadani znacky */
  printf("Zadej znacku prvku - napr. He, H, Li:\n");
  while (scanf("%s", znacka) != 1) {
    fflush(stdin);
    printf("Zadej znacku prvku - napr. He, H, Li:\n");
  }
  _setcursortype(_NOCURSOR); /* nastaveni kurzoru - zruseni */

  /* vypis vlastnosti */
  if((atom = je_atom(znacka)) != NULL) {
    fprintf(vystup,
            "\n"
            "Znacka                    : %s\n"
            "Relativni atomova hmotnost: %lf\n"
            "Protonove cislo           : %d\n"
            "Elektronegativita         : %lf\n",
            atom->znacka, atom->mr, atom->z, atom->chi);
  }
  else
    printf("\n"
           "Chyba zadani -  %s  - neni znacka prvku.\n", znacka);

  /* konec */
  printf("\n"
         "Stiskni klavesu...\n");
  (void) getkey();
}

//---------------------------PRESMEROVANI-------------------------------------
static void presmerovani(void)
  /* priradi novou hodnotu globalni promenne (pro tento modul) vystup
   * podle jeji hodnoty probihaji vytupy vypocitanych hodnot bud na
   * stdout nebo do uzivatelem specifikovaneho souboru
   */
{
  vypis_menu(p_presmer);
  zpracuj_menu(p_presmer);
}

//--------------------------NAPOVEDA------------------------------------------
static void napoveda(void)
{
  printf("Lituji, ale napoveda neni ve verzi 1.0 dostupna...\n");
  printf("\n"
         "Stiskni klavesu...\n");
  (void) getkey();
}

//-------------------------KONEC----------------------------------------------
static void konec(void)
{
  free((void *) tabulka); /* uvolneni dynamickeho pole tabulka */
  if (vystup != stdout) /* pripadne uzavreni souboru vystup */
    if (fclose(vystup) == EOF) {
      printf("\n"
             "ERROR: CHYBA PRI POKUSU O ZAPIS DAT DO SOUBORU!\n");
      (void) getkey();
    }
  exit(0); /* konec programu */
}

//-------------------------ZPET-----------------------------------------------
static void zpet(void)
{
  printf("Volba -> ZPET\n");
}

  /***************************************
   *                                     *
   * 3. Obsluhy udalosti -> menu vypocty *
   *                                     *
   ***************************************/

//---------------------------V_MR---------------------------------------------
static void v_mr(void)
  /* vypocet molarni hmotnosti latky */
{
  char vzorec[RADKA];
  double mr;

  /* uvod */
  _setcursortype(_NORMALCURSOR); /* zobrazeni kurzoru */

  /* nacteni vzorce */
  printf("Zadej vzorec latky - napr. H2SO4, HNO3, H2O:\n");
  while (scanf("%s", vzorec) != 1) {
    fflush(stdin);
    printf("Zadej vzorec latky - napr. H2SO4, HNO3, H2O:\n");
  }

  /* vypocet mr */
  if ((mr = vypocet_mr(vzorec)) == 0.0)
    printf("\n"
           "Spatne zadani -  %s  - neni chemicky vzorec...\n",
           vzorec);
  else
    fprintf(vystup,
            "\n"
            "Molarni hmotnost latky  %s  : %lf\n", vzorec, mr);

  /* zaver */
  printf("\n"
         "Stiskni klavesu...\n");
  (void) getkey();
  _setcursortype(_NOCURSOR); /* zruseni kurzoru */
}

//------------------------------V_DX------------------------------------------
static void v_dx(void)
  /* vypocet krystalograficke hustoty DX */
{
  char vzorec[RADKA];
  double mr;
  double objem;
  double dx;

  _setcursortype(_NORMALCURSOR); /* zobrazeni kurzoru */

  /* 1. KROK -> URCENI MOLARNI HMOTNOSTI LATKY */

  /* nacteni vzorce */
  printf("\n"
         "1. KROK - ZADANI LATKY...\n"
	 "\n");
  printf("Zadej pocty atomu ve vzorcove jednotce\n"
         "napr. NH3NH3NH3NH3Cu je ekvivalentni N4H12Cu:\n");
  while (scanf("%s", vzorec) != 1) {
    fflush(stdin);
    printf("Zadej pocty atomu ve vzorcove jednotce\n"
           "napr. NH3NH3NH3NH3Cu je ekvivalentni N4H12Cu:\n");
  }

  /* vypocet mr */
  if ((mr = vypocet_mr(vzorec)) == 0.0)
    printf("\n"
           "Spatne zadani -  %s  - neni vzorcova jednotka...\n",
           vzorec);

  /* 2. KROK -> URCENI OBJEMU VZORCOVE JEDNOTKY */

  else { /* mr byla spoctena -> pokracujeme */
    printf("Molarni hmotnost latky  %s  : %lf\n", vzorec, mr);

    /* vypis zahlavi 2.KROKU a menu vyp_objem */
    _setcursortype(_NOCURSOR); /* zrueni kurzoru */
    printf("\n"
           "2. KROK - URCENI OBJEMU...\n"
           "\n"
	   "Zvol soustavu, popr. zadej objem:\n");
    {
      int i;

      for (i = 0; (strcmp(p_vyp_objem->polozky[i], "") != 0); ++i)
        printf("%s\n", p_vyp_objem->polozky[i]);
    }

    /* zpracovani vyp_objem - tj vypocet objemu */
    {
      int i;
      int key;

      key = getdefined(p_vyp_objem->zkratky, &i); /* nacteni volby */
      printf("\n"
             "\tVolba: %s\n"
             "\n", p_vyp_objem->polozky[i]);
      _setcursortype(_NORMALCURSOR); /* nastaveni kurzoru - zobrazeni */
      switch (p_vyp_objem->zkratky[i]) {

        case F2:
          objem = kubic();
        break;

        case F3:
          objem = tetragonal();
        break;

        case F4:
          objem = ortorombic();
        break;

        case F5:
          objem = hexagonal();
        break;

        case F6:
          objem = romboedric();
        break;

        case F7:
          objem = monoklinic();
        break;

        case F8:
          objem = triklinic();
        break;

        case F9:
          printf("Zadej objem:\n");
          while (scanf("%lf", &objem) != 1) {
            fflush(stdin);
            printf("Zadej objem:\n");
          }
        break;

        default:
          ;
        break;
      } /* konec switch */
    } /* konec zpracovani menu */

    /* URCENI DX */
      /* v teto chvili mam zajisteno, ze vsechny vstupy od uzivatele
       * byly provedeny
       */

    _setcursortype(_NOCURSOR); /* nastaveni kurzoru - zruseni */
    printf("\n"
           "3. KROK - VYPOCET DX...\n"
           "\n");
    fprintf(vystup,
            "Molarni hmotnost vzorcove jednotky  %s: %lf\n"
	    "Objem zadane vzorcove jednotky        : %lf jednotek**3\n"
            "\n",
	    vzorec, mr, objem);

    /* vypocet a vypis DX */
    {
      int z;

      dx = vypocet_dx(mr, objem);

      for (z = 1; z <= DX; ++z)
        fprintf(vystup, "Pro z = %2d je DX = %lf\n", z, dx * z);
    }
  } /* konec else -> tj. zpracovani vypoctu objemu a DX*/

  /* KONEC */

  printf("\n"
         "Stiskni klavesu...\n");
  (void) getkey();
  _setcursortype(_NOCURSOR); /* nastaveni kurzoru - zruseni */
}

  /****************************************
   *                                      *
   * 4. Obsluhy udalosti  -> menu trideni *
   *                                      *
   ****************************************/

//----------------------------T_PODLE_ZNACKY----------------------------------
static void t_podle_znacky(void)
{
  int prvni; /* index prvniho prvku na strance */
  int posledni = 0; /* index posledniho prvku na strance */
  int i;
  int key = PGDN;
  int index;

  /* setrideni */
  q_sort(trid_znacka);

  /* vypis (vcetne vypisu a zpracovani menu t_vypis) */
   do {
    if (key == PGUP && prvni > 0) {
      prvni = ((prvni - STRANKOVANI) >= 0) ? prvni - STRANKOVANI : 0;
      posledni = prvni + STRANKOVANI;
      clrscr();
      vypis_menu(p_trid_vypis);
      printf("Por.\tZnacka\tMr\t\tProt.c.\tElektronegativita\n");
      for (i = prvni; i < posledni; ++i)
        printf("%d.\t%s\t%lf\t%d\t%lf\n",
               (i + 1), (tabulka + i)->znacka, (tabulka + i)->mr,
               (tabulka + i)->z, (tabulka + i)->chi);
    } /* konec if PGUP */
    if (key == PGDN && posledni < POCETPRVKU) {
      prvni = posledni;
      posledni = ((prvni + STRANKOVANI) < POCETPRVKU) ? prvni + STRANKOVANI : POCETPRVKU;
      clrscr();
      vypis_menu(p_trid_vypis);
      printf("Por.\tZnacka\tMr\t\tProt.c.\tElektronegativita\n");
      for (i = prvni; i < posledni; ++i)
        printf("%d.\t%s\t%lf\t%d\t%lf\n",
               (i + 1), (tabulka + i)->znacka, (tabulka + i)->mr,
               (tabulka + i)->z, (tabulka + i)->chi);
    } /* konec if PGDN */
  } while ((key = getdefined(p_trid_vypis->zkratky, &index)) != ESC);
}

//----------------------------T_PODLE_MR--------------------------------------
static void t_podle_mr(void)
{
  int prvni; /* index prvniho prvku na strance */
  int posledni = 0; /* index posledniho prvku na strance */
  int i;
  int key = PGDN;
  int index;

  /* setrideni */
  q_sort(trid_mr);

  /* vypis (vcetne vypisu a zpracovani menu t_vypis) */
   do {
    if (key == PGUP && prvni > 0) {
      prvni = ((prvni - STRANKOVANI) >= 0) ? prvni - STRANKOVANI : 0;
      posledni = prvni + STRANKOVANI;
      clrscr();
      vypis_menu(p_trid_vypis);
      printf("Por.\tZnacka\tMr\t\tProt.c.\tElektronegativita\n");
      for (i = prvni; i < posledni; ++i)
        printf("%d.\t%s\t%lf\t%d\t%lf\n",
               (i + 1), (tabulka + i)->znacka, (tabulka + i)->mr,
               (tabulka + i)->z, (tabulka + i)->chi);
    } /* konec if PGUP */
    if (key == PGDN && posledni < POCETPRVKU) {
      prvni = posledni;
      posledni = ((prvni + STRANKOVANI) < POCETPRVKU) ? prvni + STRANKOVANI : POCETPRVKU;
      clrscr();
      vypis_menu(p_trid_vypis);
      printf("Por.\tZnacka\tMr\t\tProt.c.\tElektronegativita\n");
      for (i = prvni; i < posledni; ++i)
        printf("%d.\t%s\t%lf\t%d\t%lf\n",
               (i + 1), (tabulka + i)->znacka, (tabulka + i)->mr,
               (tabulka + i)->z, (tabulka + i)->chi);
    } /* konec if PGDN */
  } while ((key = getdefined(p_trid_vypis->zkratky, &index)) != ESC);
}

//----------------------------T_PODLE_Z---------------------------------------
static void t_podle_z(void)
{
  int prvni; /* index prvniho prvku na strance */
  int posledni = 0; /* index posledniho prvku na strance */
  int i;
  int key = PGDN;
  int index;

  /* setrideni */
  q_sort(trid_z);

  /* vypis (vcetne vypisu a zpracovani menu t_vypis) */
   do {
    if (key == PGUP && prvni > 0) {
      prvni = ((prvni - STRANKOVANI) >= 0) ? prvni - STRANKOVANI : 0;
      posledni = prvni + STRANKOVANI;
      clrscr();
      vypis_menu(p_trid_vypis);
      printf("Por.\tZnacka\tMr\t\tProt.c.\tElektronegativita\n");
      for (i = prvni; i < posledni; ++i)
        printf("%d.\t%s\t%lf\t%d\t%lf\n",
               (i + 1), (tabulka + i)->znacka, (tabulka + i)->mr,
               (tabulka + i)->z, (tabulka + i)->chi);
    } /* konec if PGUP */
    if (key == PGDN && posledni < POCETPRVKU) {
      prvni = posledni;
      posledni = ((prvni + STRANKOVANI) < POCETPRVKU) ? prvni + STRANKOVANI : POCETPRVKU;
      clrscr();
      vypis_menu(p_trid_vypis);
      printf("Por.\tZnacka\tMr\t\tProt.c.\tElektronegativita\n");
      for (i = prvni; i < posledni; ++i)
        printf("%d.\t%s\t%lf\t%d\t%lf\n",
               (i + 1), (tabulka + i)->znacka, (tabulka + i)->mr,
               (tabulka + i)->z, (tabulka + i)->chi);
    } /* konec if PGDN */
  } while ((key = getdefined(p_trid_vypis->zkratky, &index)) != ESC);
}

//----------------------------T_PODLE_CHI-------------------------------------
static void t_podle_chi(void)
{
  int prvni; /* index prvniho prvku na strance */
  int posledni = 0; /* index posledniho prvku na strance */
  int i;
  int key = PGDN;
  int index;

  /* setrideni */
  q_sort(trid_chi);

  /* vypis (vcetne vypisu a zpracovani menu t_vypis) */
   do {
    if (key == PGUP && prvni > 0) {
      prvni = ((prvni - STRANKOVANI) >= 0) ? prvni - STRANKOVANI : 0;
      posledni = prvni + STRANKOVANI;
      clrscr();
      vypis_menu(p_trid_vypis);
      printf("Por.\tZnacka\tMr\t\tProt.c.\tElektronegativita\n");
      for (i = prvni; i < posledni; ++i)
        printf("%d.\t%s\t%lf\t%d\t%lf\n",
               (i + 1), (tabulka + i)->znacka, (tabulka + i)->mr,
               (tabulka + i)->z, (tabulka + i)->chi);
    } /* konec if PGUP */
    if (key == PGDN && posledni < POCETPRVKU) {
      prvni = posledni;
      posledni = ((prvni + STRANKOVANI) < POCETPRVKU) ? prvni + STRANKOVANI : POCETPRVKU;
      clrscr();
      vypis_menu(p_trid_vypis);
      printf("Por.\tZnacka\tMr\t\tProt.c.\tElektronegativita\n");
      for (i = prvni; i < posledni; ++i)
        printf("%d.\t%s\t%lf\t%d\t%lf\n",
               (i + 1), (tabulka + i)->znacka, (tabulka + i)->mr,
               (tabulka + i)->z, (tabulka + i)->chi);
    } /* konec if PGDN */
  } while ((key = getdefined(p_trid_vypis->zkratky, &index)) != ESC);
}

  /*****************************************
   *                                       *
   * Obsluhy udalosti -> menu presmerovani *
   *                                       *
   *****************************************/

//--------------------------PR_NA_OBRAZOVKU------------------------------------
static void pr_na_obrazovku(void)
{

  /* pripadne uzavreni predchoziho souboru */
  if (vystup != stdout)
    if (fclose(vystup) == EOF)
      printf("ERROR: CHYBA PRI POKUSU O ZAPIS DAT DO SOUBORU!\n");

  vystup = stdout; /* presmerovani V/V operaci */

  printf("\n"
         "Presmerovani bylo provedeno...\n"
         "Od teto chvile budou vsechny vystupy provadene v menu vypocty\n"
         "zapisovany na obazovku.\n");

  /* konec */
  printf("\n"
         "Stiskni klavesu...\n");
  (void) getkey();
}

//--------------------------PR_DO_SOUBORU--------------------------------------
static void pr_do_souboru(void)
{
  char soubor[RADKA];

  /* pripadne uzavreni predchoziho souboru */
  if (vystup != stdout)
    if (fclose(vystup) == EOF)
      printf("ERROR: CHYBA PRI POKUSU O ZAPIS DAT DO SOUBORU!\n");

  /* nacteni jmena souboru */
  _setcursortype(_NORMALCURSOR); /* nastaveni kurzoru - zruseni */
  printf("Zadej jmeno souboru, do ktereho ma byt presmerovan vystup:\n"
         "Dodrzujte prosim konvecne dane operacbim systemem.\n");
  while (scanf("%s", soubor) != 1) {
    fflush(stdin);
    printf("Zadej jmeno souboru, do ktereho ma byt presmerovan vystup:\n");
  }

  /* otevreni noveho souboru a pokud existuje pripojeni na konec
   * do souboru zapise datum a odradkuje pokud se neco pokazi vystup = stdout
   */
  if ((vystup = fopen(soubor, "a+")) == NULL) {
    printf("ERROR: CHYBA PRI POKUSU OTEVRIT SOUBOR!\n"
           "Vystup nebyl presmerovan.");
    vystup = stdout;
  }
  else {
    printf("0. Presmerovani bylo provedeno...\n"
           "1. Od teto chvile budou vsechny (spravne) vystupy provadene v menu\n"
           "   vypocty zapisovany do souboru  %s .\n"
           "2. Pokud soubor existoval, nebyl prepsan -> data budou zapisovana na\n"
           "   jeho konec. Nova data od starych budou oddelena radkou =====.\n",
           soubor);
    fprintf(vystup, "===================================================\n");
  }

  /* konec */
  _setcursortype(_NOCURSOR); /* nastaveni kurzoru - zruseni */
  printf("\n"
         "Stiskni klavesu...\n");
  (void) getkey();
}