//=========================DOKUMENTACNI CAST==================================

  /*
   * CH_TAB.C, verze 1.0
   *
   * Definuje operace nad tabulkou chemickych prvku, soucast programu CHEMTAB
   *
   * Martin Slouf, 22.4.1999
   *
   */

//========================POTREBNE INCLUDE SOUBORY============================

  /*
   * systemove .H soubory #include <stdio.h>
   *
   * vlastni .H soubory #include "vzor.h"
   *
   */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "ch_tab.h"

//===================DEFINICE GLOBALNICH IMPORTOVANYCH OBJEKTU================

  //zadne by byt nemely - vse by melo byt v .H souborech

  /*
   * jde o importovane objekty (fce nebo promenne) - jiz musi byt v nejakem
   * modulu definovany (v casti globalnich definic)
   *
   * zde se objevi jako extern deklarace
   *
   */


//=================DEFINICE GLOBALNICH EXPORTOVANYCH PROMENNYCH===============

  /*
   * jde o globalni promenne, ktere maji byt sdileny s jinymi moduly, zde
   * nejsou definovany jako extern - jde o jejich "domaci" modul
   *
   * ve sdilenych modulech se objevi za pomoci .H souboru prislusejiciho
   * k tomuto modulu (v tomto .H souboru budou uvedeny jako extern deklarace)
   *
   */

P_PRVEK tabulka;
  /* pointer na zacatek tabulky -> na prvni strukturu */

//=======================LOKALNI DEFINICE PRO TENTO MODUL=====================

  /*
   * lokalni definice pouze pro tento modul (soubor), pokud by tyto definice
   * byly prilis rozsahle -> specialni .H soubor
   *
   */

  /*
   * 1. symbolicke konstanty a makra s parametry
   *
   */

#define MAXDELKA 101
  /* maximalni delka radky pro nacteni ze souboru -> 101. znak = \0 */

  /*
   * 2. typy - zasadne pomoci typedef
   *
   */

  /*
   * 3. promenne - jde o globalni promenne pro tento modul - v pametove tride
   *    static
   *
   */

  /*
   * 4. uplne funkcni prototypy lokalnich (=pro tento modul) funkci, tj.
   * v pametove tride static
   *
   */

  /*
   * 5. fce main - pokud jde o hlavni soubor projektu
   *
   */

//========================DEFINICE GLOBALNICH FUNKCI==========================

  /*
   * jde o funkce, ktere jsou exportem modulu, jejich uplne funkcni prototypy
   * jsou v prislusnem .H souboru (obdobne jako u globalnich promennych, ani
   * zde neni uvedena zadna pametova trida, ale v prislusnem .H souboru jsou
   * deklarovany jako extern)
   *
   */

   /*
    * 1. Zakladni fce
    *
    * 2. Trideni - qsort comparison routines
    *
    */

   /*******************
    *                 *
    * 1. Zakladni fce *
    *                 *
    *******************/

//------------------------------JE_ATOM---------------------------------------
P_PRVEK je_atom(char s[])
  /* vraci pointer na atom se znackou s
   * vraci NULL pokud atom neexistuje
   */
{
  int i;

  for (i = 0; (i < POCETPRVKU && (strcmp((tabulka + i)->znacka, s) != 0)); ++i)
    ;

  if (strcmp((tabulka + i)->znacka, s) == 0)
    return(tabulka + i);
  else
    return(NULL);

  /* vhodneji */
  /*
  return(strcmp((tabulka + i)->znacka, s) == 0 ? (tabulka + i) : NULL);
  */
}

//--------------------------NACTI_DATA----------------------------------------
int nacti_data(P_PRVEK *p_prvek)
  /* vraci 1 nacetl-li data vporadku
   * vraci 0 nacetl-li data spatne
   * alokuje pamet pro celou tabulku v heapu
   * nastavi hodnotu ukazatele *p_prvek na zacatek alokovane pameti
   */
{
  char radka[MAXRADKA];
  FILE *fr;
  PRVEK prvek; /* pomocna promenna pro nacitani dat */

  /* alokace pameti pro tabulku */
  if ((*p_prvek = (P_PRVEK) malloc(POCETPRVKU * sizeof(PRVEK))) == NULL)
    return(0);

  /* otevreni souboru */
  if ((fr = fopen(SOUBOR, "rt")) == NULL)
    return(0);

  /* CTENI ZE SOUBORU */

  /* preskoceni radek s poznamkami */
  while (fgets(radka, MAXRADKA, fr) != NULL && radka[0] == '#')
    ;

  /* cteni dat */
  /* do-while ma opodstatneni -> pri vstupu do cyklu je v radce
   * ulozena prvni radka, ktera nema na zacatku '#'
   */
  do {
    /* bohuzel nelze cist primo do hodnot ukazatele double cisla - napriklad:
     * &(p_prvek->mr)
     * proto ctu nejprve do promenne prvek
     * otazkou je proc to nejde, zle i na jednoduchych prikladech jsem si
     * vyzkousel, ze to nelze
     */
    sscanf(radka,"%s %lf %d %lf", prvek.znacka, &(prvek.mr),
                                  &(prvek.z), &(prvek.chi));
    *(*p_prvek) = prvek;
    (*p_prvek)++;
  } while (fgets(radka, MAXRADKA, fr) != NULL);

  /* nastaveni pointeru na zacatek */
  *p_prvek -= POCETPRVKU;

  /* uzavreni souboru */
  if (fclose(fr) == EOF)
    return(0);

  return(1);
}

  /******************************************
   *                                        *
   * 2. Trideni - qsort comparison routines *
   *                                        *
   ******************************************/

//---------------------------TRID_ZNACKA--------------------------------------
int trid_znacka(const void *p_a, const void *p_b)
{
  P_PRVEK p_aa = (P_PRVEK) p_a;
  P_PRVEK p_bb = (P_PRVEK) p_b;

  return(strcmp(p_aa->znacka, p_bb->znacka));
}

//---------------------------TRID_MR------------------------------------------
int trid_mr(const void *p_a, const void *p_b)
{
  P_PRVEK p_aa = (P_PRVEK) p_a;
  P_PRVEK p_bb = (P_PRVEK) p_b;

  if (p_aa->mr > p_bb->mr)
    return(+1);
  else if (p_aa->mr == p_bb->mr)
    return(0);
  else
    return(-1);
}

//---------------------------TRID_Z-------------------------------------------
int trid_z(const void *p_a, const void *p_b)
{
  P_PRVEK p_aa = (P_PRVEK) p_a;
  P_PRVEK p_bb = (P_PRVEK) p_b;

  if (p_aa->z > p_bb->z)
    return(+1);
  else if (p_aa->z == p_bb->z)
    return(0);
  else
    return(-1);
}

//---------------------------TRID_CHI-----------------------------------------
int trid_chi(const void *p_a, const void *p_b)
{
  P_PRVEK p_aa = (P_PRVEK) p_a;
  P_PRVEK p_bb = (P_PRVEK) p_b;

  if (p_aa->chi > p_bb->chi)
    return(+1);
  else if (p_aa->chi == p_bb->chi)
    return(0);
  else
    return(-1);
}

//========================DEFINICE LOKALNICH FUNKCI===========================

  /*
   * definice funkci pro tento modul (pametova trida static)
   *
   * jejich funkcni prototypy - viz vyse
   *
   */